@@ -2,6 +2,8 @@ import json
 import platform
 import re
 import ssl
+import warnings
+from collections import OrderedDict
 
 import requests
 
@@ -32,11 +34,6 @@ class Client(object):
     API_ENDPOINT = 'https://api.mollie.com'
     API_VERSION = 'v2'
     UNAME = ' '.join(platform.uname())
-    USER_AGENT = ' '.join(vs.replace(r'\s+', '-') for vs in [
-        'Mollie/{client_version}'.format(client_version=CLIENT_VERSION),
-        'Python/{python_version}'.format(python_version=platform.python_version()),
-        'OpenSSL/{ssl_version}'.format(ssl_version=ssl.OPENSSL_VERSION.split(' ')[1]),
-    ])
 
     @staticmethod
     def validate_api_endpoint(api_endpoint):
@@ -50,11 +47,22 @@ class Client(object):
                 "Invalid API key: '{api_key}'. An API key must start with 'test_' or 'live_'.".format(api_key=api_key))
         return api_key
 
+    @staticmethod
+    def validate_access_token(access_token):
+        access_token = access_token.strip()
+        if not access_token.startswith('access_'):
+            raise RequestSetupError(
+                "Invalid access token: '{access_token}'. An access token must start with 'access_'.".format(
+                    access_token=access_token))
+        return access_token
+
     def __init__(self, api_key=None, api_endpoint=None, timeout=10):
         self.api_endpoint = self.validate_api_endpoint(api_endpoint or self.API_ENDPOINT)
         self.api_version = self.API_VERSION
-        self.api_key = self.validate_api_key(api_key) if api_key else None
         self.timeout = timeout
+        self.api_key = None
+
+        # add endpoint resources
         self.payments = Payments(self)
         self.payment_refunds = PaymentRefunds(self)
         self.payment_chargebacks = PaymentChargebacks(self)
@@ -68,15 +76,55 @@ class Client(object):
         self.orders = Orders(self)
         self.subscription_payments = SubscriptionPayments(self)
 
+        # compose base user agent string
+        self.user_agent_components = OrderedDict()
+        self.set_user_agent_component('Mollie', self.CLIENT_VERSION)
+        self.set_user_agent_component('Python', platform.python_version())
+        self.set_user_agent_component('OpenSSL', ssl.OPENSSL_VERSION.split(' ')[1])
+
+        if api_key:
+            # There is no clean way for supporting both API key and access token acceptance and validation
+            #  in __init__(). Furthermore the naming of the parameter would be inconsistent.
+            # Using class methods is way cleaner.
+            #
+            # Warning added in 2.1.1, remove support in 2.3.x or so.
+            msg = "Setting the API key during init will be removed in the future. " \
+                  "Use Client.set_api_key() or Client.set_access_token() instead."
+            warnings.warn(msg, PendingDeprecationWarning)
+            self.api_key = self.validate_api_key(api_key)
+
     def set_api_endpoint(self, api_endpoint):
         self.api_endpoint = self.validate_api_endpoint(api_endpoint)
 
     def set_api_key(self, api_key):
         self.api_key = self.validate_api_key(api_key)
 
+    def set_access_token(self, access_token):
+        self.api_key = self.validate_access_token(access_token)
+
     def set_timeout(self, timeout):
         self.timeout = timeout
 
+    def set_user_agent_component(self, key, value):
+        """Add or replace new user-agent component strings.
+
+        Given strings are formatted along the format agreed upon by Mollie and implementers:
+        - key and values are separated by a forward slash ("/").
+        - multiple key/values are separated by a space.
+        - keys are camel-cased, and cannot contain spaces.
+        - values cannot contain spaces.
+        """
+        key = ''.join(_x.capitalize() for _x in re.findall(r'\S+', key))
+        if re.search(r'\s+', value):
+            value = '_'.join(re.findall(r'\S+', value))
+        self.user_agent_components[key] = value
+
+    @property
+    def user_agent(self):
+        """Return the formatted user agent string."""
+        components = ["/".join(x) for x in self.user_agent_components.items()]
+        return " ".join(components)
+
     def perform_http_call(self, http_method, path, data=None, params=None):
         if not self.api_key:
             raise RequestSetupError('You have not set an API key. Please use set_api_key() to set the API key.')
@@ -104,7 +152,7 @@ class Client(object):
                     'Accept': 'application/json',
                     'Authorization': 'Bearer {api_key}'.format(api_key=self.api_key),
                     'Content-Type': 'application/json',
-                    'User-Agent': self.USER_AGENT,
+                    'User-Agent': self.user_agent,
                     'X-Mollie-Client-Info': self.UNAME,
                 },
                 params=params,
